/* Copyright (c) 2004 The Nutch Organization.  All rights reserved.   */
/* Use subject to the conditions in http://www.nutch.org/LICENSE.txt. */

package net.nutch.parse.rss;

import net.nutch.protocol.Content;
import net.nutch.util.LogFormatter;
import net.nutch.parse.Parser;
import net.nutch.parse.Parse;
import net.nutch.parse.ParseData;
import net.nutch.parse.ParseImpl;
import net.nutch.parse.Outlink;
import net.nutch.parse.ParseException;

import net.nutch.parse.rss.structs.RSSItem;
import net.nutch.parse.rss.structs.RSSChannel;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.IOException;

import java.net.MalformedURLException;

import java.util.Properties;
import java.util.logging.Logger;
import java.util.Date;
import java.util.List;
import java.util.Vector;


//add all the RSS parsing imports right here
import org.apache.commons.feedparser.FeedParserState;
import org.apache.commons.feedparser.FeedParserListener;
import org.apache.commons.feedparser.FeedParser;
import org.apache.commons.feedparser.FeedParserException;
import org.apache.commons.feedparser.FeedParserFactory;
import org.apache.commons.feedparser.DefaultFeedParserListener;
import org.apache.commons.feedparser.network.ResourceRequestFactory;
import org.apache.commons.feedparser.network.ResourceRequest;

/**
 * 
 * @author mattmann
 *
 * @author mattmann
 * @version 1.0
 *
 * <p>RSS Parser class for nutch</p>
 */
public class RSSParser implements Parser {
  public static final Logger LOG =
    LogFormatter.getLogger("net.nutch.parse.rss");

  /**
   * <p>Default Constructor</p>
   */
  public RSSParser () {

    // redirect org.apache.log4j.Logger to java's native logger, in order
    // to, at least, suppress annoying log4j warnings.
    // Note on 20040614 by Xing:
    // log4j is used by pdfbox. This snippet'd better be moved
    // to a common place shared by all parsers that use log4j.
    org.apache.log4j.Logger rootLogger =
      org.apache.log4j.Logger.getRootLogger();

    rootLogger.setLevel(org.apache.log4j.Level.INFO);

    org.apache.log4j.Appender appender = new org.apache.log4j.WriterAppender(
      new org.apache.log4j.SimpleLayout(),
      net.nutch.util.LogFormatter.getLogStream(
        this.LOG, java.util.logging.Level.INFO));

    rootLogger.addAppender(appender);
  }
    

  /**
   * <p>Implementation method, parses the RSS content, and then returns a {@link ParseImpl}.</p>
   * 
   * @param content The content to parse (hopefully an RSS content stream)
   * @return A {@link ParseImpl} which implements the {@link Parse} interface.
   */
  public Parse getParse(Content content) throws ParseException {

    // check that contentType is one we can handle
    String contentType = content.getContentType();
    if (contentType != null && (!contentType.startsWith("text/xml") && !contentType.startsWith("application/rss+xml")))
      throw new ParseException(
        "Content-Type not text/xml or application/rss+xml: "+contentType);

    List theRSSChannels = null;

    try {
      byte[] raw = content.getContent();

      String contentLength = content.get("Content-Length");

      //create a new FeedParser...
      FeedParser parser = FeedParserFactory.newFeedParser();
      

      //create a listener for handling our callbacks
      FeedParserListener listener = new FeedParserListenerImpl();

      //start parsing our feed and have the onItem methods called
      parser.parse( listener, new ByteArrayInputStream(raw), /* resource */ null);

      theRSSChannels = ((FeedParserListenerImpl)listener).getChannels();


    } catch (Exception e) { // run time exception
	e.printStackTrace();
	LOG.info("nutch:parse-rss:RSSParser Exception: "+e.getMessage());
      throw new ParseException("Can't be handled as rss document. "+e);
    } finally {
	//probably should put something here
    }

   
    String contentTitle = "", indexText = "";
    List theOutlinks = new Vector();

    //for us, the contentTitle will be a concatenation of the titles of the RSS Channels that we've parsed
    //and the index text will be a concatenation of the RSS Channel descriptions, and descriptions of the RSS Items in the channel
   
    //also get the outlinks

    if(theRSSChannels != null){
		for(int i=0; i < theRSSChannels.size(); i++){
		    RSSChannel r = (RSSChannel)theRSSChannels.get(i);
		    contentTitle+=r.getTitle()+" ";
	
		    //concat the description to the index text
		    indexText+=r.getDescription()+" ";
	
		    try{
			//get the outlink
		        theOutlinks.add(new Outlink(r.getLink(),r.getDescription()));
		    }
		    catch(MalformedURLException e){
				LOG.info("nutch:parse-rss:RSSParser Exception: MalformedURL: "+r.getLink()+": Attempting to continue processing outlinks");
				e.printStackTrace();
				continue;
		    }
	
			    //now get the descriptions of all the underlying RSS Items and then index them too
			    for(int j=0; j < r.getItems().size(); j++){
					RSSItem theRSSItem = (RSSItem)r.getItems().get(j);
					indexText+=theRSSItem.getDescription()+" ";
			
					try{
					    theOutlinks.add(new Outlink(theRSSItem.getPermalink(),theRSSItem.getDescription()));
			
					}
					catch(MalformedURLException e){
					    LOG.info("nutch:parse-rss:RSSParser Exception: MalformedURL: "+r.getLink()+": Attempting to continue processing outlinks");
					    e.printStackTrace();
					    continue;
					}
			    }
	
	       }

		LOG.info("nutch:parse-rss:getParse:indexText="+indexText);
		LOG.info("nutch:parse-rss:getParse:contentTitle="+contentTitle);

    }
    else{
		LOG.info("nutch:parse-rss:Error:getParse: No RSS Channels recorded!");
    }

    
    // collect meta data
    Properties metadata = new Properties();
    metadata.putAll(content.getMetadata()); // copy through

    //format the outlinks

    Outlink [] outlinks = (Outlink[])theOutlinks.toArray(new Outlink[theOutlinks.size()]);

    LOG.info("nutch:parse-rss:getParse:found "+outlinks.length+" outlinks");
    //LOG.info("Outlinks: "+outlinks);

    ParseData parseData = new ParseData(contentTitle, outlinks , metadata);
    return new ParseImpl(indexText, parseData);

  }


}
